<?php

namespace Internetgalerie\IgDatapoolFe\ViewHelpers\Form\Select;

use Internetgalerie\IgDatapoolFe\ViewHelpers\Form\Select2ViewHelper;
use TYPO3\CMS\Core\Utility\GeneralUtility;
use TYPO3\CMS\Extbase\Mvc\Web\Routing\UriBuilder;
use TYPO3\CMS\Extbase\Persistence\ObjectStorage;

// \Internetgalerie\IgDatapoolFe\ViewHelpers\Form\SelectViewHelper
class AjaxViewHelper extends Select2ViewHelper
{
    /**
     * Initialize arguments.
     *
     * @return void
     * @api
     */
    public function initializeArguments(): void
    {
        parent::initializeArguments();
        $this -> registerArgument('delay', 'integer', 'delay in ms', false, 250);
        $this -> registerArgument('action', 'string', 'Ajax Action', false, 'ajax');
        $this -> registerArgument('arguments', 'array', 'Ajax Arguments', false, []);
        $this -> registerArgument('controller', 'string', 'Ajax Controller', false, null);
        $this -> registerArgument('pluginName', 'string', 'Ajax PluginName', false, null);
        $this -> registerArgument('extensionName', 'string', 'Ajax ExtensionName', false, null);
        $this -> registerArgument('pageUid', 'string', 'Ajax Page UID', false, $GLOBALS['TSFE']->id);
        //$this -> registerArgument('placeholder', 'string', 'placeholder String', false, '');
        $this -> registerArgument('allowClear', 'boolean', 'display c clear button ("x" icon) to appear on the select box when a value is selected.', false, true);
    }


    /**
     * Render the tag.
     *
     * @return string rendered tag.
     * @api
     */
    public function render(): string
    {
        
        $this->createPageRendererForSelect2();
        //static::$pageRenderer->addCssFile('EXT:ig_datapool_fe/Resources/Public/Css/select2.css');

        $class= 'dp-select-ajax';


        if ($this->tag->hasAttribute('class')) {
            $this->tag->addAttribute('class', $class . ' ' . $this->tag->getAttribute('class'));
            $this->arguments['class']= $class . ' ' . $this->tag->getAttribute('class');
        } else {
            $this->tag->addAttribute('class', $class);
            $this->arguments['class']= $class;
        }
        $this->tag->addAttribute('style', 'width: 100%;');
        $functionName='initSelectAjax' . $this->getIdJs();
        $this->tag->addAttribute('data-dp-init', $functionName);
        $output = parent::render();


        $uriBuilder = GeneralUtility::makeInstance(UriBuilder::class);

        $uriBuilder->reset()->setTargetPageUid((int)$this->arguments['pageUid']);
        $uriBuilder->setCreateAbsoluteUri(true);
        $uri =$uriBuilder->uriFor($this->arguments['action'], $this->arguments['arguments'], $this->arguments['controller'], $this->arguments['extensionName'], $this->arguments['pluginName']);
        $chashPos = strrpos($uri, '&cHash=');
        if ($chashPos>0) {
            $uri = substr($uri, 0, $chashPos);
        }
        if ($this->hasArgument('placeholder')) {
            $placeholder="\nplaceholder: " . json_encode($this->arguments['placeholder']).",";
        } else {
            $placeholder='';
        }
        if ($this->hasArgument('prependOptionLabel')) {
            $allowClear='  allowClear: '. $this->arguments['allowClear']. ',
';
        } else {
            $allowClear='';
        }
        $js='var '.$functionName . ' = function($el) {
$el.select2({'.$placeholder.'
'.$allowClear.'  ajax: {
    url: '.json_encode($uri).',
    dataType: "json",
    delay: '.intval($this->arguments['delay']).',
    cache: true
  }
});
};';

        static::$pageRenderer->addJsFooterInlineCode('.ig-database-fe-' . $this->getId(), $js);


        return $output;
    }

    protected function getOptions(): array
    {
        if ($this->getPropertyValue()) {
            $propertyValue = $this->getPropertyValue();
            if ($propertyValue instanceof ObjectStorage) {
                $options = [];
                foreach ($propertyValue as $subValue) {
                    $options[$subValue->getUid()] = $subValue->__toString();
                }
                return $options;
            }
            return [ $this->getValueAttribute() => $this->getPropertyValue() ];
        }
        $value = $this->arguments['value'];
        if ($value) {
            if (is_object($value)) {
                return [ $this->getValueAttribute() => $value];
                //return [ $this->getValueAttribute() => $value->__toString()];
            }
            return [ $this->getValueAttribute() => $value];
        }
        return [];
    }

    /**
     * get identifier for non extbase objects with getUid method
     */
    protected function getValueAttribute($convertObjects = true)
    {
        $valueAttribute = parent::getValueAttribute($convertObjects);
        return is_object($valueAttribute) && method_exists($valueAttribute, 'getUid') ? $valueAttribute->getUid() : $valueAttribute;
    }
    protected function getId()
    {
        return  $this->arguments['property'];
    }
    protected function getIdJs()
    {
        return str_replace('-', '', $this->getId());
    }

    protected function convertToPlainValue($value)
    {
        if ($value instanceof ObjectStorage) {
            $ret = [];
            foreach ($value as $subValue) {
                $identifier = $this->persistenceManager->getIdentifierByObject($subValue);
                if ($identifier !== null) {
                    $ret[] = $identifier;
                }
            }
            return $ret;
        } else {
            return parent::convertToPlainValue($value);
        }
    }
}
